import os

from google_cloud.cloud_connector import GoogleStorage
from google_api.google_api import GoogleApi

from dotenv import load_dotenv

load_dotenv()  # For Windows systems

# Google API config
CLIENT_SECRET_FILE = os.getenv("CLIENT_SECRET_FILE")
API_NAME = os.getenv("API_NAME")
API_VERSION = os.getenv("API_VERSION")
GOOGLE_DRIVE_SCOPES = os.getenv("GOOGLE_DRIVE_SCOPES")

# Google storage service
GOOGLE_STORAGE_CREDENTIALS = os.getenv("GOOGLE_STORAGE_CREDENTIALS")
STORAGE_SERVICE = "STORAGE"
BIGQUERY = "BIGQUERY"
BUCKET = os.getenv("BUCKET_NAME")


def create_directory_in_drive(directory_name):
    googleApi = GoogleApi(
        client_secret_file=CLIENT_SECRET_FILE,
        api_name=API_NAME,
        api_version=API_VERSION,
        scopes=GOOGLE_DRIVE_SCOPES,
    )
    googleApi.create_directory_drive(directory_name=directory_name)


def upload_files_to_drive(folder_id, file_paths, mime_types):
    googleApi = GoogleApi(
        client_secret_file=CLIENT_SECRET_FILE,
        api_name=API_NAME,
        api_version=API_VERSION,
        scopes=GOOGLE_DRIVE_SCOPES,
    )
    googleApi.upload_files_to_drive(
        folder_id=folder_id, file_paths=file_paths, mime_types=mime_types
    )


def download_file_from_drive(file_ids, file_names, path=None):
    googleApi = GoogleApi(
        client_secret_file=CLIENT_SECRET_FILE,
        api_name=API_NAME,
        api_version=API_VERSION,
        scopes=GOOGLE_DRIVE_SCOPES,
    )
    googleApi.download_files_drive(
        file_ids=file_ids, file_names=file_names, destination_path=path
    )


def upload_file_to_bucket(bucket_name, file_to_upload, file_path):
    googleStorageClient = GoogleStorage(
        service=STORAGE_SERVICE, credentials=GOOGLE_STORAGE_CREDENTIALS
    )
    googleStorageClient.upload_file(
        bucket_name=bucket_name,
        source_file_name=file_to_upload,
        destination_file_name=file_path,
    )


def download_file_from_bucket(bucket_name, file_name, destination):
    googleStorageClient = GoogleStorage(
        service=STORAGE_SERVICE, credentials=GOOGLE_STORAGE_CREDENTIALS
    )
    googleStorageClient.download_file(
        bucket_name=bucket_name,
        file_name=file_name,
        destination_file=destination,
    )


def create_bucket(bucket_name, storage_class="STANDARD", location="us-central1"):
    googleStorageClient = GoogleStorage(
        service=STORAGE_SERVICE, credentials=GOOGLE_STORAGE_CREDENTIALS
    )
    googleStorageClient.create_bucket(bucket_name, storage_class, location)


def list_bucket_files(bucket_name):
    googleStorageClient = GoogleStorage(
        service=STORAGE_SERVICE, credentials=GOOGLE_STORAGE_CREDENTIALS
    )
    return googleStorageClient.list_files(bucket_name)



from typing import List, Optional
import os
import io
from .Google import Create_Service
from googleapiclient.http import MediaIoBaseDownload
from googleapiclient.http import MediaFileUpload


class GoogleApi:

    def __init__(
        self, client_secret_file: str, api_name: str, api_version, scopes: List[str]
    ) -> None:
        self.api_service = Create_Service(
            client_secret_file, api_name, api_version, scopes
        )

    def upload_files_to_drive(
        self, folder_id: str, file_paths: List[str], mime_types: List[str]
    ) -> None:
        for file_path, mime_type in zip(file_paths, mime_types):
            file_metadata = {"name": file_path.split("/")[-1], "parents": [folder_id]}

            media = MediaFileUpload(file_path, mime_type)

            self.api_service.files().create(
                body=file_metadata, media_body=media, fields="id"
            ).execute()

    def download_files_drive(
        self,
        file_ids: List[str],
        file_names: List[str],
        destination_path: Optional[str] = None,
    ) -> None:

        for file_id, file_name in zip(file_ids, file_names):
            request = self.api_service.files().get_media(fileId=file_id)

            fh = io.BytesIO()
            downloader = MediaIoBaseDownload(fd=fh, request=request)
            done = False

            while not done:
                status, done = downloader.next_chunk()
                print("Download progress {0}".format(status.progress() * 100))

            fh.seek(0)
            path = destination_path if destination_path else "./"
            with open(os.path.join(path, file_name), "wb") as f:
                f.write(fh.read())
                f.close()

    def create_directory_drive(self, directory_name: str):
        file_metadata = {
            "name": directory_name,
            "mimeType": "application/vnd.google-apps.folder",
        }

        self.api_service.files().create(body=file_metadata).execute()



import os

from google_cloud.cloud_connector import GoogleBigQuery
from dotenv import load_dotenv

load_dotenv()  # For Windows systems

# Google API config
CLIENT_SECRET_FILE = os.getenv("CLIENT_SECRET_FILE")
API_NAME = os.getenv("API_NAME")
API_VERSION = os.getenv("API_VERSION")
GOOGLE_DRIVE_SCOPES = os.getenv("GOOGLE_DRIVE_SCOPES")

# Google storage service
GOOGLE_BIGQUEY_CREDENTIALS = os.getenv("GOOGLE_BIGQUEY_CREDENTIALS")
BIGQUERY = "BIGQUERY"
BUCKET = os.getenv("BUCKET_NAME")


def sql_query(query):
    bigquery = GoogleBigQuery(service=BIGQUERY, credentials=GOOGLE_BIGQUEY_CREDENTIALS)
    sql_job = bigquery.sql_query(query=query)
    for item in sql_job:
        print(item)


def add_labels_in_table(labels, dataset_name, table_name):
    bigquery = GoogleBigQuery(service=BIGQUERY, credentials=GOOGLE_BIGQUEY_CREDENTIALS)
    bigquery.add_table_labels(
        labels=labels, dataset_name=dataset_name, table_name=table_name
    )


def delete_labels_in_table(dataset_name, table_name):
    bigquery = GoogleBigQuery(service=BIGQUERY, credentials=GOOGLE_BIGQUEY_CREDENTIALS)
    bigquery.delete_table_labels(dataset_name=dataset_name, table_name=table_name)


def save_query_as_table(query, destination_dataset_name, destination_table_name):
    bigquery = GoogleBigQuery(service=BIGQUERY, credentials=GOOGLE_BIGQUEY_CREDENTIALS)
    bigquery.save_query_as_table(
        query=query,
        destination_dataset_name=destination_dataset_name,
        destination_table_name=destination_table_name,
    )


def export_queries_to_excel(queries):
    # TODO! make constants for scopes services
    scopes = [
        "https://www.googleapis.com/auth/cloud-platform",
        "https://www.googleapis.com/auth/drive",
        "https://www.googleapis.com/auth/bigquery",
    ]
    bigquery = GoogleBigQuery(
        service=BIGQUERY, credentials=GOOGLE_BIGQUEY_CREDENTIALS, scopes=scopes
    )
    bigquery.export_queries_to_excel(queries=queries)



import pickle
import os
from google_auth_oauthlib.flow import Flow, InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.http import MediaFileUpload, MediaIoBaseDownload
from google.auth.transport.requests import Request
import datetime


def Create_Service(client_secret_file, api_name, api_version, *scopes):
    CLIENT_SECRET_FILE = client_secret_file
    API_SERVICE_NAME = api_name
    API_VERSION = api_version
    SCOPES = [scope for scope in scopes[0]]

    cred = None

    pickle_file = f"token_{API_SERVICE_NAME}_{API_VERSION}.pickle"
    # print(pickle_file)

    if os.path.exists(pickle_file):
        with open(pickle_file, "rb") as token:
            cred = pickle.load(token)

    if not cred or not cred.valid:
        if cred and cred.expired and cred.refresh_token:
            cred.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(CLIENT_SECRET_FILE, SCOPES)
            cred = flow.run_local_server()

        with open(pickle_file, "wb") as token:
            pickle.dump(cred, token)

    try:
        service = build(API_SERVICE_NAME, API_VERSION, credentials=cred)
        print(API_SERVICE_NAME, "service created successfully")
        return service
    except Exception as e:
        print("Unable to connect.")
        print(e)
        return None


def convert_to_RFC_datetime(year=1900, month=1, day=1, hour=0, minute=0):
    dt = datetime.datetime(year, month, day, hour, minute, 0).isoformat() + "Z"
    return dt



import json
from google.oauth2.service_account import Credentials

import time
from typing import List, Any, Tuple, Dict, Optional
from google.cloud import storage
from google.cloud.storage import Bucket
from google.cloud import bigquery
import os
#from ..tools.files_manage import insert_data_in_df


class GoogleCloud:

    def __init__(
        self, service: str, credentials: str, scopes: Optional[List[str]] = None
    ) -> None:
        if service == "STORAGE":
            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = credentials
            self.storage_client = storage.Client()
        elif service == "BIGQUERY":
            credentials_s = Credentials.from_service_account_file(
                credentials, scopes=scopes
            )
            self.bigquery_client = bigquery.Client(credentials=credentials_s)
        else:
            messege = "Undefined the service {}".format(service)
            raise Exception(messege)


class GoogleStorage(GoogleCloud):

    def create_bucket(
        self, bucket_name: str, storage_class="STANDARD", location="us-central1"
    ) -> Bucket:
        try:
            bucket = self.storage_client.bucket(bucket_name)
            bucket.storage_class = storage_class

            bucket = self.storage_client.create_bucket(bucket, location)
            return bucket
        except Exception as e:
            print(e)

            return None

    def upload_file(
        self, bucket_name: str, source_file_name: str, destination_file_name: str
    ) -> bool:
        try:
            bucket = self.storage_client.bucket(bucket_name)

            blob = bucket.blob(destination_file_name)
            blob.upload_from_filename(source_file_name)

            return True
        except Exception as e:
            print(e)

            return False

    def download_file(
        self, bucket_name: str, file_name: str, destination_file: str
    ) -> bool:
        try:
            bucket = self.storage_client.bucket(bucket_name)

            blob = bucket.blob(file_name)
            blob.download_to_filename(destination_file)

            return True
        except Exception as e:
            print(e)

            return False

    def list_files(self, bucket_name: str) -> List[str]:
        try:
            file_list = self.storage_client.list_blobs(bucket_name)
            return [file.name for file in file_list]
        except Exception as e:
            print(e)

            return []


class GoogleBigQuery(GoogleCloud):

    def sql_query(self, query: str):  # TODO: See type of object
        return self.bigquery_client.query(query)

    def table_reference(self, dataset_name: str, table_name: str) -> Tuple[Any, Any]:
        data_reference = bigquery.DatasetReference(
            self.bigquery_client.project, dataset_name
        )
        table_reference = bigquery.TableReference(data_reference, table_name)

        return data_reference, table_reference

    def add_table_labels(self, labels: Dict, dataset_name: str, table_name: str) -> Any:
        _, table_reference = self.table_reference(
            dataset_name=dataset_name, table_name=table_name
        )
        table = self.bigquery_client.get_table(table_reference)
        table.labels = labels
        return self.bigquery_client.update_table(table, ["labels"])

    def delete_table_labels(self, dataset_name: str, table_name: str) -> Any:
        _, table_reference = self.table_reference(
            dataset_name=dataset_name, table_name=table_name
        )
        table = self.bigquery_client.get_table(table_reference)
        labels = table.labels
        table.labels = {k: None for k, v in labels.items()}
        return self.bigquery_client.update_table(table, ["labels"])

    def save_query_as_table(
        self, query: str, destination_dataset_name: str, destination_table_name: str
    ):
        destination_dataset_reference = self.bigquery_client.dataset(
            destination_dataset_name, self.bigquery_client.project
        )
        destination_dataset = self.bigquery_client.get_dataset(
            destination_dataset_reference
        )

        destination_table_reference = destination_dataset_reference.table(
            destination_table_name
        )

        query_job_config = bigquery.QueryJobConfig()
        query_job_config.destination = destination_table_reference

        query_job = self.bigquery_client.query(
            query=query,
            location=destination_dataset.location,
            job_config=query_job_config,
        )

        while query_job.state != "DONE":
            time.sleep(1)
            query_job.reload()

    '''def export_queries_to_excel(self, queries: List[str]):
        xl_app = win32.Dispatch("Excel.Application")
        xl_app.Visible = True
        wb = xl_app.Workbooks.Add()

        for query in queries:
            query_job = self.bigquery_client.query(query)
            while query_job.state != "DONE":
                time.sleep(1)
                query_job.reload()
            response = query_job.result(max_results=None)
            dataframe = response.to_dataframe()
            worksheet = wb.Worksheets.Add()
            insert_data_in_df(worksheet=worksheet, dataframe=dataframe)'''



def insert_data_in_df(worksheet, dataframe):
    # Insert header
    worksheet.Range(
        worksheet.Cells(1, 1), worksheet.Cells(1, dataframe.columns.shape[0])
    ).Value = dataframe.columns
    dataframe = dataframe.fillna(0)
    worksheet.Range(
        worksheet.Cells(2, 1),
        worksheet.Cells(dataframe.shape[0] + 1, dataframe.shape[1]),
    ).Value = dataframe.values



from examples.sorage_and_drive import *
from examples.bigquery import *

if __name__ == "__main__":
    command = str(input("Enter command: "))

    if command == "upload_to_drive":
        folder_id = "1XFeGaXDL9D6xGbw40jJ2FUMLzG0LY9BJ"
        file_paths = ["G:/Proyectos/Python/Libs/wj-google/files/dark_dragon.jpg"]
        mime_types = ["image/jpeg"]
        upload_files_to_drive(folder_id, file_paths, mime_types)

    if command == "download_from_drive":
        # File id to download from Drive and the name of the file in local
        file_ids = ["121La8gl0HURGIk-_pmJMOjT9G4KsSR0b"]
        file_names = ["image.jpg"]
        download_file_from_drive(file_ids, file_names)

    if command == "download_from_drive_with_path":
        # File id to download from Drive and the name of the file in local
        file_ids = ["121La8gl0HURGIk-_pmJMOjT9G4KsSR0b"]
        file_names = ["image.jpg"]
        path = "d:/Descargas"
        download_file_from_drive(file_ids, file_names, path)

    if command == "create_directory_in_drive":
        directory_name = "test"
        create_directory_in_drive(directory_name)

    if command == "upload_to_bucket":
        # File downloaded from  Drive and path for bucket
        FILE_TO_UPLOAD = "G:/Proyectos/Python/Libs/wj_google/files/image.jpg"
        FILE_PATH = "files/image.jpg"
        bucket_name = "jcmantilla_bucket"
        upload_file_to_bucket(bucket_name, FILE_TO_UPLOAD, FILE_PATH)

    if command == "download_from_bucket":
        bucket_name = "jcmantilla_bucket"
        file_name = "files/image.jpg"
        destination = "G:/Proyectos/Python/Libs/wj_google/files/image.jpg"
        download_file_from_bucket(bucket_name, file_name, destination)

    if command == "create_bucket":
        bucket_name = "jcmantilla_test_bucket"
        create_bucket(bucket_name)

    if command == "list_bucket_files":
        bucket_name = "jcmantilla_bucket"
        print(list_bucket_files(bucket_name))

    if command == "bigquery_consult":
        query = """
         SELECT * FROM `test.tw` LIMIT 10
        """
        query_job = sql_query(query)

    if command == "add_table_label_bigquery":
        dataset_name = "dt_OCEANA_Holberton"
        table_name = "test_Holberton"
        labels = {
            "type": "social_media",
            "category": "cloud",
        }
        add_labels_in_table(labels, dataset_name, table_name)

    if command == "delete_table_label_bigquery":
        dataset_name = "dt_OCEANA_Holberton"
        table_name = "test_Holberton"
        delete_labels_in_table(dataset_name, table_name)

    if command == "save_query_as_table":
        query = """
        SELECT * FROM `thematic-cider-416220.dt_OCEANA_Holberton.test_Holberton` LIMIT 100
        """
        destination_dataset_name = "reports"
        destination_table_name = "test_table"
        save_query_as_table(query, destination_dataset_name, destination_table_name)

    '''if command == "export_querys_to_Excel":
        query1 = """
        SELECT * FROM `test.tw` LIMIT 10
        """
        query2 = """
        SELECT * FROM `reports.reports` LIMIT 10
        """
        queries = [query1, query2]
        export_queries_to_excel(queries=queries)'''
